@@ -35,7 +35,7 @@
 #include "gtestutils.h"
 
 /**
- * SECTION: linked_lists_double
+ * SECTION:linked_lists_double
  * @title: Doubly-Linked Lists
  * @short_description: linked lists containing integer values or
  *                     pointers to data, with the ability to iterate
@@ -48,7 +48,7 @@
  * pointers which link to the previous and next elements in the list.
  * Using these pointers it is possible to move through the list in both
  * directions (unlike the <link
- * linkend="glib-Singly-Linked-lists">Singly-Linked Lists</link> which
+ * linkend="glib-Singly-Linked-Lists">Singly-Linked Lists</link> which
  * only allows movement through the list in the forward direction).
  *
  * The data contained in each element can be either integer values, by
@@ -212,10 +212,15 @@ g_list_free_1 (GList *list)
  */
 void
 g_list_free_full (GList          *list,
-		  GDestroyNotify  free_func)
+                  GDestroyNotify  free_func)
 {
-  g_list_foreach (list, (GFunc) free_func, NULL);
-  g_list_free (list);
+  while (list)
+    {
+      GList *next = list->next;
+      (*free_func) (list->data);
+      _g_list_free1 (list);
+      list = next;
+    }
 }
 
 /**
