@@ -931,7 +931,7 @@ g_ascii_strtoll (const gchar *nptr,
  *     is unknown, it returns "unknown error (&lt;code&gt;)". The string
  *     can only be used until the next call to g_strerror()
  */
-G_CONST_RETURN gchar*
+const gchar *
 g_strerror (gint errnum)
 {
   static GStaticPrivate msg_private = G_STATIC_PRIVATE_INIT;
@@ -1412,7 +1412,7 @@ g_strerror (gint errnum)
  *     it returns "unknown signal (&lt;signum&gt;)". The string can only be
  *     used until the next call to g_strsignal()
  */
-G_CONST_RETURN gchar*
+const gchar *
 g_strsignal (gint signum)
 {
   static GStaticPrivate msg_private = G_STATIC_PRIVATE_INIT;
@@ -2966,7 +2966,7 @@ g_str_has_prefix (const gchar  *str,
  *
  * Since: 2.4
  **/
-G_CONST_RETURN gchar *
+const gchar *
 g_strip_context  (const gchar *msgid,
                   const gchar *msgval)
 {
@@ -3032,7 +3032,7 @@ g_strv_length (gchar **str_array)
  *
  * Since: 2.16
  */
-G_CONST_RETURN gchar *
+const gchar *
 g_dpgettext (const gchar *domain,
              const gchar *msgctxtid,
              gsize        msgidoffset)
@@ -3093,7 +3093,7 @@ g_dpgettext (const gchar *domain,
  *
  * Since: 2.18
  */
-G_CONST_RETURN char *
+const char *
 g_dpgettext2 (const char *domain,
               const char *msgctxt,
               const char *msgid)
@@ -3214,7 +3214,7 @@ _g_dgettext_should_translate (void)
  *
  * Since: 2.18
  */
-G_CONST_RETURN gchar *
+const gchar *
 g_dgettext (const gchar *domain,
             const gchar *msgid)
 {
@@ -3240,7 +3240,7 @@ g_dgettext (const gchar *domain,
  *
  * Since: 2.26
  */
-G_CONST_RETURN gchar *
+const gchar *
 g_dcgettext (const gchar *domain,
              const gchar *msgid,
              int          category)
@@ -3270,7 +3270,7 @@ g_dcgettext (const gchar *domain,
  *
  * Since: 2.18
  */
-G_CONST_RETURN gchar *
+const gchar *
 g_dngettext (const gchar *domain,
              const gchar *msgid,
              const gchar *msgid_plural,
